@@ -1,11 +1,10 @@
-{-# LANGUAGE TypeFamilies
-           , FlexibleInstances
-           , FlexibleContexts
-           , UndecidableInstances
-           , GeneralizedNewtypeDeriving
-           , StandaloneDeriving
-           , MultiParamTypeClasses
-  #-}
+{-# LANGUAGE FlexibleContexts           #-}
+{-# LANGUAGE FlexibleInstances          #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE TypeFamilies               #-}
+{-# LANGUAGE UndecidableInstances       #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Graphics.Rendering.Diagrams.Envelope
@@ -43,13 +42,13 @@ module Diagrams.Core.Envelope
        , OrderedField
        ) where
 
-import           Control.Applicative ((<$>))
-import qualified Data.Map as M
-import           Data.Maybe (fromMaybe)
+import           Control.Applicative     ((<$>))
+import qualified Data.Map                as M
+import           Data.Maybe              (fromMaybe)
 import           Data.Semigroup
-import qualified Data.Set as S
+import qualified Data.Set                as S
 
-import           Data.AffineSpace ((.+^), (.-^))
+import           Data.AffineSpace        ((.+^), (.-^))
 import           Data.VectorSpace
 
 import           Diagrams.Core.HasOrigin
@@ -185,6 +184,9 @@ instance (InnerSpace v, OrderedField (Scalar v)) => Enveloped (Envelope v) where
 instance (OrderedField (Scalar v), InnerSpace v) => Enveloped (Point v) where
   getEnvelope p = moveTo p . mkEnvelope $ const zeroV
 
+instance Enveloped t => Enveloped (TransInv t) where
+  getEnvelope = getEnvelope . unTransInv
+
 instance (Enveloped a, Enveloped b, V a ~ V b) => Enveloped (a,b) where
   getEnvelope (x,y) = getEnvelope x <> getEnvelope y
 
@@ -251,7 +253,7 @@ envelopeS v = fromMaybe 0 . envelopeSMay v
 --   vector.  Returns zero for the empty envelope.
 diameter :: Enveloped a => V a -> a -> Scalar (V a)
 diameter v a = case appEnvelope $ getEnvelope a of
-  (Just env) -> (env v - env (negateV v)) * magnitude v
+  (Just env) -> (env v + env (negateV v)) * magnitude v
   Nothing -> 0
 
 -- | Compute the \"radius\" (1\/2 the diameter) of an enveloped object
