@@ -1,17 +1,16 @@
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE TypeFamilies #-}
-{-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE GADTs #-}
-{-# LANGUAGE ExistentialQuantification #-}
-{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE DeriveDataTypeable         #-}
+{-# LANGUAGE EmptyDataDecls             #-}
+{-# LANGUAGE ExistentialQuantification  #-}
+{-# LANGUAGE FlexibleContexts           #-}
+{-# LANGUAGE FlexibleInstances          #-}
+{-# LANGUAGE GADTs                      #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE TypeOperators #-}
-{-# LANGUAGE OverlappingInstances #-}
-{-# LANGUAGE UndecidableInstances #-}
-{-# LANGUAGE TupleSections #-}
-{-# LANGUAGE EmptyDataDecls #-}
+{-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE ScopedTypeVariables        #-}
+{-# LANGUAGE TupleSections              #-}
+{-# LANGUAGE TypeFamilies               #-}
+{-# LANGUAGE TypeOperators              #-}
+{-# LANGUAGE UndecidableInstances       #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -59,12 +58,11 @@ module Diagrams.Core.Types
 
          -- ** Modifying diagrams
          -- *** Names
-       , named
        , nameSub
-       , namePoint
        , withName
        , withNameAll
        , withNames
+       , localize
 
          -- *** Other
        , freeze
@@ -87,7 +85,7 @@ module Diagrams.Core.Types
          -- * Primtives
          -- $prim
 
-       , Prim(..), nullPrim
+       , Prim(..), IsPrim(..), nullPrim
 
          -- * Backends
 
@@ -104,16 +102,16 @@ module Diagrams.Core.Types
 
        ) where
 
-import           Control.Applicative ((<$>), (<*>))
-import           Control.Arrow (first, second, (***))
-import           Control.Monad (mplus)
+import           Control.Applicative       ((<$>), (<*>))
+import           Control.Arrow             (first, second, (***))
+import           Control.Monad             (mplus)
 import           Control.Newtype
-import           Data.AffineSpace ((.-.))
-import           Data.List (isSuffixOf)
-import qualified Data.Map as M
-import           Data.Maybe (listToMaybe, fromMaybe)
+import           Data.AffineSpace          ((.-.))
+import           Data.List                 (isSuffixOf)
+import qualified Data.Map                  as M
+import           Data.Maybe                (fromMaybe, listToMaybe)
 import           Data.Semigroup
-import qualified Data.Traversable as T
+import qualified Data.Traversable          as T
 import           Data.Typeable
 import           Data.VectorSpace
 
@@ -123,7 +121,7 @@ import           Data.Monoid.Deletable
 import           Data.Monoid.MList
 import           Data.Monoid.Split
 import           Data.Monoid.WithSemigroup
-import qualified Data.Tree.DUAL as D
+import qualified Data.Tree.DUAL            as D
 
 import           Diagrams.Core.Envelope
 import           Diagrams.Core.HasOrigin
@@ -160,7 +158,7 @@ import           Diagrams.Core.V
 --   * query functions (see "Diagrams.Core.Query")
 type UpAnnots b v m = Deletable (Envelope v)
                   ::: Deletable (Trace v)
-                  ::: SubMap b v m
+                  ::: Deletable (SubMap b v m)
                   ::: Query v m
                   ::: ()
 
@@ -265,32 +263,18 @@ setTrace t = over QD ( D.applyUpre (inj . toDeletable $ t)
 -- | Get the subdiagram map (/i.e./ an association from names to
 --   subdiagrams) of a diagram.
 subMap :: QDiagram b v m -> SubMap b v m
-subMap = getU' . unQD
+subMap = unDelete . getU' . unQD
 
 -- | Get a list of names of subdiagrams and their locations.
 names :: HasLinearMap v => QDiagram b v m -> [(Name, [Point v])]
 names = (map . second . map) location . M.assocs . unpack . subMap
 
--- | Attach an atomic name to a diagram.
-named :: ( IsName n
-         , HasLinearMap v, InnerSpace v, OrderedField (Scalar v), Semigroup m)
-      => n -> QDiagram b v m -> QDiagram b v m
-named = nameSub mkSubdiagram
-
--- | Attach an atomic name to a certain point (which may be computed
---   from the given diagram), treated as a subdiagram with no content
---   and a point envelope.
-namePoint :: ( IsName n
-             , HasLinearMap v, InnerSpace v, OrderedField (Scalar v), Semigroup m)
-          => (QDiagram b v m -> Point v) -> n -> QDiagram b v m -> QDiagram b v m
-namePoint p = nameSub (subPoint . p)
-
 -- | Attach an atomic name to a certain subdiagram, computed from the
 --   given diagram.
 nameSub :: ( IsName n
            , HasLinearMap v, InnerSpace v, OrderedField (Scalar v), Semigroup m)
         => (QDiagram b v m -> Subdiagram b v m) -> n -> QDiagram b v m -> QDiagram b v m
-nameSub s n d = over QD (D.applyUpre . inj $ fromNames [(n,s d)]) d
+nameSub s n d = over QD (D.applyUpre . inj . toDeletable $ fromNames [(n,s d)]) d
 
 -- | Given a name and a diagram transformation indexed by a
 --   subdiagram, perform the transformation using the most recent
@@ -321,6 +305,17 @@ withNames :: IsName n
 withNames ns f d = maybe id f (T.sequence (map ((listToMaybe=<<) . ($nd) . lookupSub . toName) ns)) d
   where nd = subMap d
 
+-- | \"Localize\" a diagram by hiding all the names, so they are no
+--   longer visible to the outside.
+localize :: forall b v m. ( HasLinearMap v, InnerSpace v, OrderedField (Scalar v)
+                          , Semigroup m
+                          )
+         => QDiagram b v m -> QDiagram b v m
+localize = over QD ( D.applyUpre  (inj (deleteL :: Deletable (SubMap b v m)))
+                   . D.applyUpost (inj (deleteR :: Deletable (SubMap b v m)))
+                   )
+
+
 -- | Get the query function associated with a diagram.
 query :: Monoid m => QDiagram b v m -> Query v m
 query = getU' . unQD
@@ -351,7 +346,8 @@ clearValue = fmap (const (Any False))
 -- | Create a diagram from a single primitive, along with an envelope,
 --   trace, subdiagram map, and query function.
 mkQD :: Prim b v -> Envelope v -> Trace v -> SubMap b v m -> Query v m -> QDiagram b v m
-mkQD p e t n q = QD $ D.leaf (toDeletable e *: toDeletable t *: n *: q *: ()) p
+mkQD p e t n q
+  = QD $ D.leaf (toDeletable e *: toDeletable t *: toDeletable n *: q *: ()) p
 
 ------------------------------------------------------------
 --  Instances
@@ -395,7 +391,7 @@ infixl 6 `atop`
 
 instance Functor (QDiagram b v) where
   fmap f = (over QD . D.mapU . second . second)
-             ( (first . fmap . fmap) f
+             ( (first . fmap . fmap . fmap) f
              . (second . first . fmap . fmap) f
              )
 
@@ -624,8 +620,14 @@ rememberAs n b = over SubMap $ M.insertWith (++) (toName n) [mkSubdiagram b]
 instance Action Name (SubMap b v m) where
   act = (|>)
 
--- | Names don't act on anything else.
-instance Action Name a
+instance Action Name a => Action Name (Deletable a) where
+  act n (Deletable l a r) = Deletable l (act n a) r
+
+-- Names do not act on other things.
+
+instance Action Name (Query v m)
+instance Action Name (Envelope v)
+instance Action Name (Trace v)
 
 -- | Look for the given name in a name map, returning a list of
 --   subdiagrams associated with that name.  If no names match the
@@ -651,13 +653,25 @@ lookupSub a (SubMap m)
 -- the collection of primitives a given backend knows how to render is
 -- determined by instances of 'Renderable'.
 
+-- | A type class for primitive things which know how to handle being
+--   transformed by both a normal transformation and a \"frozen\"
+--   transformation.  The default implementation simply applies both.
+--   At the moment, 'ScaleInv' is the only type with a non-default
+--   instance of 'IsPrim'.
+class Transformable p => IsPrim p where
+  transformWithFreeze :: Transformation (V p) -> Transformation (V p) -> p -> p
+  transformWithFreeze t1 t2 = transform (t1 <> t2)
+
 -- | A value of type @Prim b v@ is an opaque (existentially quantified)
 --   primitive which backend @b@ knows how to render in vector space @v@.
 data Prim b v where
-  Prim :: Renderable p b => p -> Prim b (V p)
+  Prim :: (IsPrim p, Renderable p b) => p -> Prim b (V p)
 
 type instance V (Prim b v) = v
 
+instance HasLinearMap v => IsPrim (Prim b v) where
+  transformWithFreeze t1 t2 (Prim p) = Prim $ transformWithFreeze t1 t2 p
+
 -- | The 'Transformable' instance for 'Prim' just pushes calls to
 --   'transform' down through the 'Prim' constructor.
 instance HasLinearMap v => Transformable (Prim b v) where
@@ -673,6 +687,8 @@ data NullPrim v = NullPrim
 
 type instance (V (NullPrim v)) = v
 
+instance HasLinearMap v => IsPrim (NullPrim v)
+
 instance HasLinearMap v => Transformable (NullPrim v) where
   transform _ _ = NullPrim
 
@@ -751,7 +767,7 @@ class (HasLinearMap v, Monoid (Render b v)) => Backend b v where
               = withStyle b s mempty (render b (transform t p))
 
             renderOne (p, (t1 :| t2, s))
-              = withStyle b s t1 (render b (transform (t1 <> t2) p))
+              = withStyle b s t1 (render b (transformWithFreeze t1 t2 p))
 
   -- See Note [backend token]
 
